import java.awt.*;
import java.awt.event.*;

public class Planet extends Panel implements AdjustmentListener, Runnable {

    private int[] pxt = new int[720];
    private int[] pyt = new int[720];

    private int a,b,c;
    private int ox,oy,px,py;
    private int step,time;
    private int periodo = 6000;    
    private double e = 0.7;
    private double am;
    private double olde;

    private Scrollbar scroll;
    private Label messLabel;
    private Image bi;
    private Graphics bg;
    private Dimension dbDim;
    private Thread thread;

    public Planet() {
		scroll = new Scrollbar(Scrollbar.HORIZONTAL,70,0,1,100);
		messLabel = new Label("Eccentricit = " + format(e), Label.CENTER);
		scroll.addAdjustmentListener(this);
		messLabel.setFont( new Font("monospaced", Font.PLAIN, 12) );
		scroll.setBounds(100, 7, 120, 16);
		messLabel.setBounds(240, 5, 150, 20);
        setLayout( new BorderLayout() );
		Panel p = new Panel();
		p.setLayout(null);
		p.setSize(400, 30);
        p.add(scroll   );
        p.add(messLabel);
		add(p, BorderLayout.SOUTH );
		calcola();
		thread = new Thread(this);
		thread.start();
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
        int valore = scroll.getValue();
        e=(double)(valore)/100.;
        messLabel.setText("Eccentricit = " + format(e));
        repaint();
    }

    public String format(double i) {
		Double d = new Double(i);
		String s = d.toString();
		while(s.length() < 6) s += " ";
		return s;
    }

    private void getDefaults() {
        b = (int)(Math.sqrt(a*a*(1.-e*e)));
		c = (int)(a*e);
		if(e != olde) calcola();
    }

    private void calcola() {
		for(int i=0; i<720; i++) {
			am=2*Math.PI*i/720;
            double aecc = anomaliaEccentrica(am, e);
            pxt[i] = (int)(a*Math.cos(aecc));
            pyt[i] = (int)(b*Math.sin(aecc));
		}
    }

    public void run() {
		getDefaults();
		time=(int)(System.currentTimeMillis());
        while(true) {
			step = (int)(System.currentTimeMillis()-time);
			step %= periodo;
			step = (int)(step*720/periodo);
			px = ox + pxt[step];
            py = oy - pyt[step];
            repaint();
			try{
				Thread.sleep(10);
			}catch(InterruptedException e) {
			}
        }
    }

    private double anomaliaEccentrica(double m,double ecc) {
		double e0 = m;
		double e1, olde1;
		double eps = 1.e-06;
		do{
			e1 = e0+(m+ecc*Math.sin(e0)-e0)/(1-ecc*Math.cos(e0));
			olde1 = e0;
			e0 = e1;        
		}while(Math.abs(e1-olde1)>eps);
		return e1;
    }

    public void paint(Graphics g) {
		update(g);
    }

    public void update(Graphics g) {
		dbDim = getSize();
		ox = dbDim.width/2;
        oy = dbDim.height/2 - 15;
        a = Math.min(ox,oy)-10;
		if (bg == null) {
			bi = createImage(dbDim.width, dbDim.height);
			bg = bi.getGraphics();
		}
		getDefaults();
		bg.setColor(Color.white);
		bg.fillRect(0,0,dbDim.width, dbDim.height);
		bg.setColor(Color.blue);
		bg.drawOval(ox-a,oy-b,2*a,2*b);
		bg.setColor(Color.red);
		bg.drawLine(ox-a,oy,ox+a,oy);
		bg.drawLine(ox,oy+b,ox,oy-b);
		bg.setColor(Color.magenta);
		bg.drawLine(ox+c,oy-10,ox+c,oy+10);
		bg.setColor(Color.orange);
		bg.fillOval(px-5,py-5,10,10);
		g.drawImage(bi, 0, 0, this);
    }

    public static void main(String args[]){
		Frame frame = new Frame();
		frame.setTitle( "Moto Kepleriano" );
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e){System.exit(0);}
		});
		Planet plt=new Planet();
		frame.add(plt,"Center");
		int w=430, h=450;
		frame.setSize(w,h);
		frame.setResizable(false);
		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		frame.setLocation( (d.width-w)/2, (d.height-h)/2 );
		frame.show();
    }    
}